class SBNGProtectPlayerName extends NexgenPlugin config (SBNGProtectPlayerName);


enum EPTState {
	PS_Never,
	PS_Inactive,
	PS_Active,
};


struct TProtectionInfo {
	var string 	PlayerName;
	var int		EndDate; // format "yyyymmdd"
	var string	NexgenClientID;
};
var config TProtectionInfo ProtectionInfos[256];
var int ProtectionCount; // number of first valid entries in array Sponsors
var config bool bSpawnProtectPlayerNameInfo; // Spawn an extra info for each player so that 3rd party can access the data?
var config name ProtectPlayerNameInfoSpawnedEvent; // raise this event when a *ProtectPlayerNameInfo* was spawned
var config bool bDebuglog;

var config string MutateCmd_ProtectionState;
var config string MutateProtectionMsg_Active;
var config string MutateProtectionMsg_Inactive;
var config string MutateProtectionMsg_None;



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a client is attempting to login. This allows to plugin to accept or
 *                reject the login request. If the function returns false the login request will be
 *                rejected (player will be disconnected). Please make sure the reason parameter
 *                is set in that case, as it will be written to the log.
 *  $PARAM        client      Client that is requesting to login to the server.
 *  $PARAM        rejectType  Reject type identification code.
 *  $PARAM        reason      Message describing why the login is rejected.
 *  $PARAM        popupWindowClass  Class name of the popup window that is to be shown at the client.
 *  $PARAM        popupArgs         Optional arguments for the popup window. Note you may have to
 *                                  explicitly reset them if you change the popupWindowClass.
 *  $REQUIRE      client != none
 *  $RETURN       True if the login request is accepted, false if it should be rejected.
 *  $ENSURE       result == false ? new.reason != "" : true
 *
 **************************************************************************************************/
function bool checkLogin(NexgenClient client, out name rejectType, out string reason, out string popupWindowClass, out string popupArgs[6]) {

// *reason* is logged in server's log, not at client
	//printPlayerID("checkLogin", client);
	if ( ! LoginAllowed(client))
	{
		reason = "usage of reserved name";
		//rejectType = 'ReservedName';
		popupWindowClass = "NexgenLoginFailedDialog";
		//popupWindowClass = "NexgenIllegalLoginDialog";
		// changing default values has no effect here
		//class'NexgenIllegalLoginDialog'.Default.Message = "The server has disconnected your client because the name you have chosen is reserved. If you continue to experience this problem please report it to the administrator of this server.";
		//class'NexgenIllegalLoginDialog'.Default.Caption = "Failed to login: that name is reserved.";
		//popupWindowClass = "NexgenReservedNameDialog";
		popupArgs[0] = "Use another name, because this one is reserved: '" $ client.PlayerName $ "'";
		return false;
	}

	return true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called whenever a client has finished its initialisation process. During this
 *                process things such as the remote control window are created. So only after the
 *                client is fully initialized all functions can be safely called.
 *  $PARAM        client  The client that has finished initializing.
 *  $REQUIRE      client != none
 *
 **************************************************************************************************/
function clientInitialized(NexgenClient client) {
	if (bSpawnProtectPlayerNameInfo)
		SpawnProtectPlayerNameInfo(client);

}



function int CompactProtectionInfos() {
local int i, j, result;

	for (i = 0; i < ArrayCount(ProtectionInfos); i++)
		if (ProtectionInfos[i].PlayerName == "")
		{
			for (j = i + 1; j < ArrayCount(ProtectionInfos); j++)
				if (ProtectionInfos[j].PlayerName != "")
				{
					ProtectionInfos[i].PlayerName = ProtectionInfos[j].PlayerName;
					ProtectionInfos[i].EndDate = ProtectionInfos[j].EndDate;
					ProtectionInfos[i].NexgenClientID = ProtectionInfos[j].NexgenClientID;
					result++;
					ProtectionInfos[j].PlayerName = "";
					ProtectionInfos[j].EndDate = 0;
					ProtectionInfos[j].NexgenClientID = "";
					break;
				}
		}
		else
			result++;
	return result;
}



function DebugLog(string FunctionName, string msg) {
	if (bDebuglog)
		log(self $ "." $ FunctionName $ "():" @ msg);
}



function string GetProtectionInfoString(TProtectionInfo ProtectionInfo) {
	return "PlayerName='" $ ProtectionInfo.PlayerName $ "', EndDate=" $ string(ProtectionInfo.EndDate) $ ", NexgenClientID=" $ ProtectionInfo.NexgenClientID;
}



function int GetTimeStamp() {
// returns the current date in format YYYYMMDD
	return Level.Day + Level.Month * 100 + Level.Year * 10000;
}



function EPTState GetProtectionState(int ISODateInt) {

	if (ISODateInt <= 0)
		return PS_Never;
	else if (GetTimeStamp() > ISODateInt)
		return PS_Inactive;
	else
		return PS_Active;
}



function bool HaveProtectedNexgenclient(NexgenClient client, out int Index) {
/******************************************************************************
Returns TRUE if *client* is in the list *ProtectionInfos* regardless of its
*EndDate*. *Index* is set to the list item index.
******************************************************************************/
local int i;

	for (i = 0; i < ProtectionCount; i++)
		if (client.playerID == ProtectionInfos[i].NexgenClientID)
		{
			index = i;
			return true;
		}
	return false;
}



function bool ISODateToDate(int ISODate, out int year, out byte month, out byte day) {
/******************************************************************************
Expects a date in ISO format (YYYYMMDD) in ISODate and sets *year*, *month*
and *day* accordingly.

Returns TRUE if a rudimentary date check succeeds.
******************************************************************************/
local string s;

	s = string(ISODate);
	year = int(left(s ,4));
	month = byte(mid(ISODate, 4, 2));
	day = byte(right(ISODate, 2));
	return year >=1000 && year <= 9999 && month > 0 && month <= 12 && day > 0 && day <= 31;
}



function string ISODateToString(int ISODate, string FormatStr) {
local byte day, month;
local int year;

	ISODateToDate(ISODate, year, month, day);
	FormatStr = class'NexgenUtil'.static.replace(FormatStr, "%YYYY%", year);
	FormatStr = class'NexgenUtil'.static.replace(FormatStr, "%MM%", TwoDigitString(month));
	FormatStr = class'NexgenUtil'.static.replace(FormatStr, "%M%", month);
	FormatStr = class'NexgenUtil'.static.replace(FormatStr, "%DD%", TwoDigitString(day));
	FormatStr = class'NexgenUtil'.static.replace(FormatStr, "%D%", day);
	return FormatStr;
}



function bool LoginAllowed(NexgenClient client) {
/******************************************************************************
Returns TRUE, if:

******************************************************************************/

local int i;

	for (i = 0; i < ProtectionCount; i++)
		if (client.playerName ~= ProtectionInfos[i].PlayerName)
			if (ProtectionValid(ProtectionInfos[i].EndDate))
				return client.playerID == ProtectionInfos[i].NexgenClientID;

	return true;
}



event LogProtectionList(int amount) {
local int i;

	//log("ProtectionCount=" $ ProtectionCount);
	//log("GetTimeStamp()=" $ GetTimeStamp());
	amount = min(amount, ArrayCount(ProtectionInfos));
	DebugLog("LogProtectionList", "listing" @ amount @ "entries of ProtectionInfo list:");
	for (i = 0; i < amount; i++)
		DebugLog("LogProtectionList", "ProtectionInfos[" $ i $ "]:" @ GeTProtectionInfoString(ProtectionInfos[i]));
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a player has send a mutate call to the server.
 *  $PARAM        mutateString  Mutator specific string (indicates the action to perform).
 *  $PARAM        sender        Player that has send the message.
 *
 **************************************************************************************************/
function mutate(string mutateString, PlayerPawn sender) {
local int i, ISODateInt;
local NexgenClient client;
local EPTState ProtectionState;

	if (mutateString ~= MutateCmd_ProtectionState)
	{
		if (NexgenClient_Find(sender, client))
		{
			if (HaveProtectedNexgenclient(client, i))
			{
				ISODateInt = ProtectionInfos[i].EndDate;
				ProtectionState = GetProtectionState(ISODateInt);
			}
			else
				ProtectionState = PS_Never;

			switch (ProtectionState)
			{
				case PS_Never:
					Sender.ClientMessage(ISODateToString(ISODateInt, MutateProtectionMsg_None));
					break;
				case PS_Inactive:
					Sender.ClientMessage(ISODateToString(ISODateInt, MutateProtectionMsg_Inactive));
					break;
				case PS_Active:
					Sender.ClientMessage(ISODateToString(ISODateInt, MutateProtectionMsg_Active));
					break;
				default:
					Sender.ClientMessage("unkown status:" @ ProtectionState);
			}
		}
		else
			Sender.ClientMessage("error retrieving status");
	}
}



function bool NexgenClient_Find(PlayerPawn Player, out NexgenClient client) {
/******************************************************************************
Returns TRUE if a NexgenClient was found and sets *client* accordingly.
******************************************************************************/

	ForEach AllActors(class'NexgenClient', Client)
		if (Client.Player == Player)
			return true;
	return false;
}



event PostBeginPlay() {

	ProtectionCount = CompactProtectionInfos();
	if (bDebuglog)
		LogProtectionList(ProtectionCount);
	Super.PostBeginPlay();
}



event printPlayerID(string FunctionName, NexgenClient client) {
	log("SBNGProtectPlayerName." $ FunctionName $ "(): client.playerID=" $ client.playerID);
}



function bool ProtectionValid(int ISODate) {
	return GetTimeStamp() <= ISODate;
}



function bool SpawnProtectPlayerNameInfo(NexgenClient client) {
local ProtectPlayerNameInfo PPNInfo;
local int i;
local Actor A;

	if (HaveProtectedNexgenclient(client, i))
	{
		PPNInfo = spawn(class'ProtectPlayerNameInfo', client.player);
		if (PPNInfo != None)
		{
			PPNInfo.Player = client.Player;
			PPNInfo.ProtectionEnddate = ProtectionInfos[i].EndDate;
			PPNInfo.NexgenClientID = ProtectionInfos[i].NexgenClientID;
			PPNInfo.bProtectionActive = ProtectionValid(ProtectionInfos[i].EndDate);
			A = client.Player;
			// log("Setting TAG to" @ A.Name);
			PPNInfo.Tag = A.Name;
			// log("PPNInfo.Tag is now" @ PPNInfo.Tag $ ", PPNInfo.bProtectionActive=" $ PPNInfo.bProtectionActive);
			if (bDebuglog)
				log(self $ ": spawned" @ PPNInfo @ "with Tag='" $ PPNInfo.Tag $ "'");
			if (ProtectPlayerNameInfoSpawnedEvent != '')
				foreach AllActors(class'Actor', A, ProtectPlayerNameInfoSpawnedEvent)
				{
					A.Trigger(PPNInfo, client.Player);
					DebugLog("SpawnProtectPlayerNameInfo", "risen event '" $ ProtectPlayerNameInfoSpawnedEvent $ "' for" @ A);
				}
			return true;
		}
		else
			warn("could not spawn" @ PPNInfo.class.name);
	}
	else
		DebugLog("SpawnProtectPlayerNameInfo", "Player" @ client.Player @ "not found in list *ProtectionInfos*");
	return false;
}



static function string TwoDigitString(int Num) {
	if (Num >= 0 && Num < 10)
		return "0" $ string(Num);
	else
		return string(Num);
}



defaultproperties {
	bDebuglog=false
	bSpawnProtectPlayerNameInfo=false
	ProtectPlayerNameInfoSpawnedEvent="ProtectPlayerNameInfoSpawned"
	MutateCmd_ProtectionState="ProtectionState"
	MutateProtectionMsg_Active="Your name protection is valid until %YYYY%-%MM%-%DD% (format YYYY-MM-DD)"
	MutateProtectionMsg_Inactive="Your name protection has run out at %YYYY%-%MM%-%DD% (format YYYY-MM-DD)"
	MutateProtectionMsg_None="You don't have a name protection"
	pluginName="SBNGProtectPlayerName"
	pluginAuthor="SeriousBarbie@barbies.world"
	pluginVersion="0"
}